import cv2
import matplotlib.pyplot as plt


def resize(image, width=None, height=None):
    """
    Resize an image to a specified width and height.
    :return: the resized image
    """

    return image


def erode(image, kernel_size=5):
    """
    Erode an image.
    :return: the eroded image
    """

    return image


def dilate(image, kernel_size=5):
    """
    Dilate an image.
    :return: the dilated image
    """

    return image


def blur_median(image, kernel_size=5):
    """
    Blur an image using median filter.
    :return: the blurred image
    """

    return image


def blur_gaussian(image, kernel_size=5):
    """
    Blur an image using gaussian filter.
    :return: the blurred image
    """

    return image


def blur_bilateral(image, kernel_size=5):
    """
    Blur an image using bilateral filter.
    :return: the blurred image
    """

    return image


def histogram(image):
    """
    Calculate the histogram of an image.
    :return: the histogram of the image
    """

    return image


def equalize_histogram(image):
    """
    Equalize the histogram of an image.
    :return: the equalized image
    """

    return image


def threshold(image, threshold=128):
    """
    Threshold an image.
    :return: the thresholded image
    """

    return image


def threshold_binary(image, threshold=128):
    """
    Threshold an image to binary.
    :return: the thresholded image
    """

    return image


def threshold_adaptive(image, threshold=128):
    """
    Threshold an image using adaptive thresholding.
    :return: the thresholded image
    """

    return image


def threshold_otsu(image):
    """
    Threshold an image using Otsu's method.
    :return: the thresholded image
    """

    return image

def morphological_open(image, kernel_size=5):
    """
    Morphological open an image.
    :return: the opened image
    """

    return image

def morphological_close(image, kernel_size=5):
    """
    Morphological close an image.
    :return: the closed image
    """

    return image

def morphological_gradient(image, kernel_size=5):
    """
    Morphological gradient an image.
    :return: the gradient image
    """

    return image

if __name__ == "__main__":
    image = cv2.imread("images/lena.png", cv2.IMREAD_COLOR)
    image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    resized = resize(image, width=300, height=300)
    erode = erode(image, kernel_size=5)
    dilate = dilate(image, kernel_size=5)

    f, plot = plt.subplots(2, 2, figsize=(15, 15))
    plot[0, 0].imshow(image)
    plot[0, 0].set_title("Original")
    plot[0, 1].imshow(resized)
    plot[0, 1].set_title("Resized")
    plot[1, 0].imshow(erode)
    plot[1, 0].set_title("Eroded")
    plot[1, 1].imshow(dilate)
    plot[1, 1].set_title("Dilated")
    plt.show()

    gausian = blur_gaussian(image, kernel_size=5)
    median = blur_median(image, kernel_size=5)
    bilateral = blur_bilateral(image, kernel_size=5)
    f, plot = plt.subplots(2, 2, figsize=(15, 15))
    plot[0, 0].imshow(image)
    plot[0, 0].set_title("Original")
    plot[0, 1].imshow(gausian)
    plot[0, 1].set_title("Gaussian")
    plot[1, 0].imshow(median)
    plot[1, 0].set_title("Median")
    plot[1, 1].imshow(bilateral)
    plot[1, 1].set_title("Bilateral")
    plt.show()

    normal = threshold(image, threshold=128)
    binary = threshold_binary(image, threshold=128)
    adaptive = threshold_adaptive(image, threshold=128)
    otsu = threshold_otsu(image)

    f, plot = plt.subplots(2, 2, figsize=(15, 15))
    plot[0, 0].imshow(normal)
    plot[0, 0].set_title("Simple Threshold")
    plot[0, 1].imshow(binary)
    plot[0, 1].set_title("Binary Threshold")
    plot[1, 0].imshow(adaptive)
    plot[1, 0].set_title("Adaptive Threshold")
    plot[1, 1].imshow(otsu)
    plot[1, 1].set_title("Otsu Threshold")
    plt.show()

    hist = histogram(image)
    equalized = equalize_histogram(image)
    hist_equalized = histogram(equalized)
    f, plot = plt.subplots(2, 2, figsize=(15, 15))
    plot[0, 0].imshow(image)
    plot[0, 0].set_title("Original")
    plot[0, 1].imshow(hist)
    plot[0, 1].set_title("Histogram")
    plot[1, 0].imshow(equalized)
    plot[1, 0].set_title("Equalized")
    plot[1, 1].imshow(hist_equalized)
    plot[1, 1].set_title("Histogram Equalized")
    plt.show()

    opened = morphological_open(image, kernel_size=5)
    closed = morphological_close(image, kernel_size=5)
    gradient = morphological_gradient(image, kernel_size=5)
    f, plot = plt.subplots(2, 2, figsize=(15, 15))
    plot[0, 0].imshow(image)
    plot[0, 0].set_title("Original")
    plot[0, 1].imshow(opened)
    plot[0, 1].set_title("Opened")
    plot[1, 0].imshow(closed)
    plot[1, 0].set_title("Closed")
    plot[1, 1].imshow(gradient)
    plot[1, 1].set_title("Gradient")
    plt.show()
